import { Component, ContentChild, EventEmitter, Injectable, Input, NgModule, Output } from '@angular/core';
import { CommonModule } from '@angular/common';

const CUSTOM_PATTERNS = {
    'url': {
        type: 'url',
        pattern: /(https?:\/\/|www\.)[-a-zA-Z0-9@:%._\+~#=]{1,256}\.[a-z]{2,6}\b([-a-zA-Z0-9@:%_\+.~#?&\/=]*[-a-zA-Z0-9@:%_\+~#?&\/=])*/i,
        element: {
            tag: '<a>',
            attributes: {
                target: '_blank',
                /**
                 * @param {?} link
                 * @param {?} $event
                 * @return {?}
                 */
                displayText(link, $event) {
                    return link;
                },
                /**
                 * @param {?} link
                 * @param {?} $event
                 * @return {?}
                 */
                href(link, $event) {
                    return link;
                },
            },
        },
    },
    'phone': {
        type: 'phone',
        pattern: /[\+]?[(]?[0-9]{3}[)]?[-\s\.]?[0-9]{3}[-\s\.]?[0-9]{4,7}/,
        element: {
            tag: '<a>',
            attributes: {
                target: '_self',
                /**
                 * @param {?} link
                 * @param {?} $event
                 * @return {?}
                 */
                displayText(link, $event) {
                    return link;
                },
                /**
                 * @param {?} link
                 * @param {?} $event
                 * @return {?}
                 */
                href(link, $event) {
                    return `tel:${link}`;
                },
            },
        },
    },
    'email': {
        type: 'email',
        pattern: /\S+@\S+\.\S+/,
        element: {
            tag: '<a>',
            attributes: {
                target: '_blank',
                /**
                 * @param {?} link
                 * @param {?} $event
                 * @return {?}
                 */
                displayText(link, $event) {
                    return link;
                },
                /**
                 * @param {?} link
                 * @param {?} $event
                 * @return {?}
                 */
                href(link, $event) {
                    return `mailto:${link}`;
                },
            },
        },
    },
};
class HighlightService {
    /**
     * @param {?} patterns
     * @return {?}
     */
    linkPatterns(patterns) {
        const /** @type {?} */ checkedPatterns = [];
        const /** @type {?} */ types = {};
        patterns.forEach((pat) => {
            if (pat.pattern instanceof RegExp) {
                checkedPatterns.push(pat);
                if (CUSTOM_PATTERNS[pat.type]) {
                    types[pat.type] = true;
                }
            }
        });
        Object.keys(CUSTOM_PATTERNS).forEach((type) => {
            if (!types[type]) {
                checkedPatterns.push(CUSTOM_PATTERNS[type]);
            }
        });
        this.PATTERNS = [...checkedPatterns];
    }
    /**
     * @param {?} text
     * @return {?}
     */
    parse(text) {
        let /** @type {?} */ hyperText = [{ text, match: false }], /** @type {?} */ str = '';
        this.PATTERNS.forEach((patterns) => {
            const /** @type {?} */ parts = [];
            hyperText.forEach((text) => {
                if (text.match) {
                    parts.push(text);
                    return;
                }
                str = text.text;
                while (patterns.pattern.test(str)) {
                    const /** @type {?} */ matches = patterns.pattern.exec(str);
                    const /** @type {?} */ index = matches.index;
                    const /** @type {?} */ left = str.substr(0, index - 1);
                    parts.push({ text: left, match: false });
                    const /** @type {?} */ link = str.substr(index, matches[0].length);
                    parts.push(this.setParsing(patterns, true, link));
                    str = str.slice(index + matches[0].length + 1);
                }
                if (str) {
                    parts.push({ text: str, match: false });
                }
            });
            hyperText = parts;
        });
        return hyperText;
    }
    /**
     * @param {?} patterns
     * @param {?} match
     * @param {?} link
     * @return {?}
     */
    setParsing(patterns, match, link) {
        let /** @type {?} */ setTag, /** @type {?} */ setAttributes = {};
        const { type, pattern, element: { tag, attributes }, } = patterns;
        setTag = !patterns.element && (!tag || tag !== '<a>') ? '<span>' : tag;
        if (typeof attributes === 'object') {
            ['target', 'href', 'displayText'].forEach((attr) => {
                if (attributes[attr] !== undefined) {
                    if (typeof attributes[attr] === 'function') {
                        setAttributes[attr] = attributes[attr](link, { type, pattern });
                    }
                    else {
                        setAttributes[attr] = attributes[attr];
                    }
                }
            });
        }
        if (Object.keys(setAttributes).length === 0) {
            setTag = '<span>';
            setAttributes = { displayText: link };
        }
        return Object.assign({ type,
            pattern,
            match,
            link, tag: setTag }, setAttributes);
    }
}
HighlightService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
HighlightService.ctorParameters = () => [];

class ParsedTextComponent {
    /**
     * @param {?} highlight
     */
    constructor(highlight) {
        this.highlight = highlight;
        this.trigger = new EventEmitter();
    }
    /**
     * @param {?} text
     * @return {?}
     */
    set text(text) {
        this._text = text;
        this.highlightText();
    }
    /**
     * @param {?} patterns
     * @return {?}
     */
    set parse(patterns) {
        this._parse = patterns;
        this.highlightText();
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.highlightText();
    }
    /**
     * @return {?}
     */
    highlightText() {
        if (!this._text || typeof this._text !== 'string') {
            console.error('text is empty or not in correct format');
            return;
        }
        let /** @type {?} */ patterns = [];
        if (Array.isArray(this._parse)) {
            patterns = this._parse;
        }
        this.highlight.linkPatterns(patterns);
        this.parsedList = this.highlight.parse(this._text);
    }
    /**
     * @param {?} parse
     * @return {?}
     */
    emitLink(parse) {
        this.trigger.emit(parse);
    }
}
ParsedTextComponent.decorators = [
    { type: Component, args: [{
                selector: 'ng-parsed-text',
                template: `
  <ng-template #link let-parse>
    <a
      class="link"
      *ngIf="parse.tag === '<a>'"
      [href]="parse.href"
      [target]="parse.target">
        {{parse.displayText}}
    </a>
    <a
      class="link clickable"
      *ngIf="parse.tag !== '<a>'"
      (click)="emitLink(parse)"
    >
        {{parse.link}}
    </a>
  </ng-template>
  <ng-template ngFor let-item [ngForOf]="parsedList"]>
    <ng-container *ngIf="item.match  else renderText">
      <ng-container 
        [ngTemplateOutlet]="custom ? custom : link"
        [ngTemplateOutletContext]="{$implicit: item}"
      >
      </ng-container>
    </ng-container>
    <ng-template #renderText>
      {{ item.text }}
    </ng-template>
  </ng-template>`,
                styles: [
                    `.link {
    cursor: pointer;
    color: #2196f3;
    text-decoration: none;
  }
  :host-context(.ng-parse-text-red) .link {
      color: #de1919;
  }
  :host-context(.ng-parse-text-green) .link {
      color: #7ebb38
  }`
                ],
            },] },
];
/**
 * @nocollapse
 */
ParsedTextComponent.ctorParameters = () => [
    { type: HighlightService, },
];
ParsedTextComponent.propDecorators = {
    'text': [{ type: Input },],
    'parse': [{ type: Input },],
    'trigger': [{ type: Output },],
    'custom': [{ type: ContentChild, args: ['custom',] },],
};

class ParsedTextModule {
}
ParsedTextModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [ParsedTextComponent],
                exports: [ParsedTextComponent],
                providers: [HighlightService]
            },] },
];
/**
 * @nocollapse
 */
ParsedTextModule.ctorParameters = () => [];

/**
 * Generated bundle index. Do not edit.
 */

export { ParsedTextModule, ParsedTextComponent, HighlightService };
//# sourceMappingURL=ng-parsed-text.js.map
